@@ -171,7 +171,19 @@ typedef enum _GstQuarkId
   GST_QUARK_ELEMENT_METADATA_KLASS = 142,
   GST_QUARK_ELEMENT_METADATA_DESCRIPTION = 143,
   GST_QUARK_ELEMENT_METADATA_AUTHOR = 144,
-  GST_QUARK_MAX = 145
+  GST_QUARK_TOC = 145,
+  GST_QUARK_TOC_ENTRY = 146,
+  GST_QUARK_UPDATED = 147,
+  GST_QUARK_EXTEND_UID = 148,
+  GST_QUARK_TOC_UID = 149,
+  GST_QUARK_TAGS = 150,
+  GST_QUARK_SUB_ENTRIES = 151,
+  GST_QUARK_INFO = 152,
+  GST_QUARK_INFO_STRUCTURE = 153,
+  GST_QUARK_TIME_STRUCTURE = 154,
+  GST_QUARK_MESSAGE_TAG = 155,
+  GST_QUARK_EVENT_TAG = 156,
+  GST_QUARK_MAX = 157
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
